/********************************** (C) COPYRIGHT *******************************
* File Name          : mainDriver.c
* Author             : WCH
* Version            : V1.0
* Date               : 2021/08/09
* Description        :
*******************************************************************************/


#include "usb_regs.h"
#include "USBDevice.h"
#include "usb_desc.h"
#include "debug.h"
union{
    __attribute__ ((aligned(4)))  uint8_t USBOTG_EPxBuf[64*3];   //ep0(64) ep2_out(64)+ep2_in(64)
//    __attribute__ ((aligned(16))) uint8_t USBHS_EPxBuf[512*3]; //ep0(64)
    __attribute__ ((aligned(4)))  uint8_t USBD_EPxBuf[64*3];
}USBBuf;



#define  USBD_pEP0_INOUTBuf         (&USBBuf.USBD_EPxBuf[0])
#define  USBD_pEP2_OUTBuf           (&USBBuf.USBD_EPxBuf[64])
#define  USBD_pEP2_INBuf            (&USBBuf.USBD_EPxBuf[64*2])
#define  pSetupReqPakUSBD           ((PUSB_SETUP_REQ)USBD_pEP0_INOUTBuf)

uint8_t  *pDescr;
uint8_t   DevConfig;
uint8_t   SetupReqCode;
uint8_t   DevAddress = 0x00;
uint16_t  SetupReqLen;

UINT8 UartCfg[ 8 ];

t_sendUsbData sendUsbData = {0,0,0,0,0,0,0};

//UINT8V testWaitSendCount = 0;

const UINT8 usbFrameByteNum[12] = {0, 32, 32, 28, 28, 24, 28, 32, 20, 22, 24, 26};
const UINT8 needSendLen[12] = {0, USB_NEEDSEND_LEN1, USB_NEEDSEND_LEN2, USB_NEEDSEND_LEN3, USB_NEEDSEND_LEN4, USB_NEEDSEND_LEN5,
        USB_NEEDSEND_LEN6, USB_NEEDSEND_LEN7, USB_NEEDSEND_LEN8, USB_NEEDSEND_LEN9, USB_NEEDSEND_LEN10, USB_NEEDSEND_LEN11};

/* VCPر */
#define DEF_VEN_I2C_CMD_X                 0X54                                  /* I2Cӿڵ,ִ */
#define DEF_VEN_DELAY_MS                  0X5E                                  /* Ϊλʱָʱ */
#define DEF_VEN_GET_VER                   0X5F                                  /* ȡоƬ汾 */
#define DEF_VEN_UART0_INIT                0XA1                                  /* ʼ0 */
#define DEF_VEN_UART1_INIT                0XA2                                  /* ʼ1 */
#define DEF_VEN_UART0_M_OUT               0XA4                                  /* MODEM0ź,bit7-4:AFE/RTS/DTR/OUT */
#define DEF_VEN_UART1_M_OUT               0XA5                                  /* MODEM1ź,bit7-4:AFE/RTS/DTR/OUT */
#define DEF_VEN_UART2_INIT                0XB1                                  /* ʼ2 */
#define DEF_VEN_UART3_INIT                0XB2                                  /* ʼ3 */
#define DEF_VEN_UART2_M_OUT               0XB4                                  /* MODEM2ź,bit7-4:AFE/RTS/DTR/OUT */
#define DEF_VEN_UART3_M_OUT               0XB5                                  /* MODEM3ź,bit7-4:AFE/RTS/DTR/OUT */
#define DEF_VEN_GPIO_RD                   0XA9                                  /* GPIOȡ(GPIO11GPIO0ķ򼰵ƽ״̬) */
#define DEF_VEN_GPIO_EN_WR                0XAA                                  /* GPIOʹܼ/ */
#define DEF_VEN_GPIO_IO_WR                0XAB                                  /* GPIOƽ */
#define DEF_VEN_GPIO_EN_RD                0XAD                                  /* GPIOʹܼ/״̬ȡ */

/* CDC */
#define DEF_GET_LINE_CODING           0X21                                      /* This request allows the host to find out the currently configured line coding */
#define DEF_SET_LINE_CODING           0x20                                      /* Configures DTE rate, stop-bits, parity, and number-of-character */
#define DEF_SET_LINE_CTLSTE           0X22                                      /* This request generates RS-232/V.24 style control signals */
#define DEF_SEND_BREAK                0X23                                      /* Sends special carrier modulation used to specify RS-232 style break */

/*******************************************************************************
* Function Name  : USBD_Endp_Init
* Description    : USB˵ʼ
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void USBD_Endp_Init( void )
{
    *( EP0REG + ENDP0 )= EP_CONTROL | ENDP0;                                    /* ep0 ctrl */
    SetEPRxAddr( ENDP0, USBD_ENDP0_RXADDR );
    SetEPTxAddr( ENDP0, USBD_ENDP0_TXADDR );
    SetEPRxCount( ENDP0, 64 );
    SetEPRxStatus( ENDP0, EP_RX_VALID );

    *( EP0REG + ENDP1 )= EP_INTERRUPT | ENDP1;
    SetEPRxAddr( ENDP1, USBD_ENDP1_RXADDR );
    SetEPTxAddr( ENDP1, USBD_ENDP1_TXADDR );
    SetEPRxCount( ENDP1, 64 );
    ClearDTOG_RX( ENDP1 );
    ClearDTOG_TX( ENDP1 );
    SetEPRxStatus( ENDP1, EP_RX_DIS );
    SetEPTxStatus( ENDP1 ,EP_TX_NAK );

    *( EP0REG + ENDP2 )= EP_BULK | ENDP2;                                 /* ep2 ctrl */
    SetEPRxAddr( ENDP2, USBD_ENDP2_RXADDR );
    SetEPTxAddr( ENDP2, USBD_ENDP2_TXADDR );
    SetEPRxCount( ENDP2, 64 );
    ClearDTOG_RX( ENDP2 );
    ClearDTOG_TX( ENDP2 );
    SetEPRxStatus( ENDP2, EP_RX_VALID );
    SetEPTxStatus( ENDP2 ,EP_TX_NAK );
}

/*******************************************************************************
* Function Name  : UserToUSBDBufferCopy
* Description    : ûݿUSB
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void USBDevClose(void)
{
    _SetDADDR( 0 );
    _SetCNTR(0);
    _SetCNTR( CNTR_FRES | CNTR_PDWN );

	RCC_APB1PeriphClockCmd(RCC_APB1Periph_USB,DISABLE);    //ʱ
    RCC_APB1PeriphResetCmd(RCC_APB1Periph_USB, ENABLE);
    RCC_APB1PeriphResetCmd(RCC_APB1Periph_USB, DISABLE);

    NVIC_DisableIRQ(USB_LP_CAN1_RX0_IRQn);
    NVIC_DisableIRQ(USBWakeUp_IRQn);
}

void UserToUSBDBufferCopy(uint8_t *pbUsrBuf, uint16_t wPMABufAddr, uint16_t wNBytes);

/*******************************************************************************
* Function Name  : UserToUSBDBufferCopy
* Description    : ûݿUSB
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void USBDInit( void )
{
    RCC_USBCLKConfig( RCC_USBCLKSource_PLLCLK_Div3 );
    RCC_APB1PeriphClockCmd( RCC_APB1Periph_USB, ENABLE );
    SetCNTR( CNTR_FRES );
    SetCNTR( 0 );
    SetENDPOINT( 0, _GetENDPOINT( 0 ) & 0x7F7F & EPREG_MASK );
    SetENDPOINT( 1, _GetENDPOINT( 1 ) & 0x7F7F & EPREG_MASK );
    SetENDPOINT( 2, _GetENDPOINT( 2 ) & 0x7F7F & EPREG_MASK );
    SetENDPOINT( 3, _GetENDPOINT( 3 ) & 0x7F7F & EPREG_MASK );
    SetENDPOINT( 4, _GetENDPOINT( 4 ) & 0x7F7F & EPREG_MASK );
    SetENDPOINT( 5, _GetENDPOINT( 5 ) & 0x7F7F & EPREG_MASK );
    SetENDPOINT( 6, _GetENDPOINT( 6 ) & 0x7F7F & EPREG_MASK );
    SetENDPOINT( 7, _GetENDPOINT( 7 ) & 0x7F7F & EPREG_MASK );
    SetDADDR( 0 | DADDR_EF );
    SetISTR( 0 );
    SetCNTR( CNTR_CTRM  | CNTR_WKUPM | CNTR_SUSPM | CNTR_ERRM | CNTR_RESETM );
    SetCNTR( _GetCNTR( ) | CNTR_PDWN ); // power down
    Delay_Ms( 1 );
    SetCNTR( _GetCNTR( )&( ~CNTR_PDWN ) );
    EXTEN->EXTEN_CTR |= EXTEN_USBD_PU_EN;

}

/*******************************************************************************
* Function Name  : UserToUSBDBufferCopy
* Description    : ûݿUSB
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void UserToUSBDBufferCopy( uint8_t *pbUsrBuf, uint16_t wPMABufAddr, uint16_t wNBytes )
{
    uint32_t i;
    uint32_t *pdwVal;

	pdwVal = (uint32_t *)(wPMABufAddr * 2 + PMAAddr);
	for( i = ( wNBytes + 1 ) >> 1; i != 0; i-- )
	{
    	*pdwVal++ = *((uint16_t *)pbUsrBuf);
		pbUsrBuf += 2;
    }
}

/*******************************************************************************
* Function Name  : USBDToUserBufferCopy
* Description    : USBݿû
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void USBDToUserBufferCopy( uint8_t *pbUsrBuf, uint16_t wPMABufAddr, uint16_t wNBytes )
{
	uint32_t i;
	uint32_t *pdwVal;

	pdwVal = (uint32_t *)( wPMABufAddr * 2 + PMAAddr );
	for( i = ( wNBytes + 1 ) >> 1; i != 0; i-- )  
	{
		*((uint16_t *)pbUsrBuf ) = *pdwVal++;
		pbUsrBuf += 2;
	}
}
/*******************************************************************************
* Function Name  : USB_LP_CAN1_RX0_IRQHandler
* Description    : USBжϺ
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void USB_LP_CAN1_RX0_IRQHandler(void) __attribute__((interrupt("WCH-Interrupt-fast")));
void USB_LP_CAN1_RX0_IRQHandler( void )
{
	union
	{
		uint8_t* b;
		uint16_t* w;
	}pBuf;
	uint16_t  intflag;
	uint8_t   EPindex, errflag = 0;
	uint8_t   len;
	uint16_t  wEPVal, wCNTR;
	uint8_t   buf[ 5 ];

	intflag = _GetISTR( );

	if( intflag & ISTR_CTR )
	{  

		/* Correct TRansfer (clear-only bit) */
		SetISTR((uint16_t)CLR_CTR );
		EPindex = (uint8_t)( intflag & ISTR_EP_ID );
        wEPVal = _GetENDPOINT( EPindex );
		if( EPindex == 2 )
		{
			if( wEPVal & EP_CTR_RX )
			{  
				/* ep2-OUT */
                ClearEP_CTR_RX( EPindex );
                /* ת浽ͻ */
                len = GetEPRxCount( EPindex );
                USBDToUserBufferCopy(USBD_pEP2_OUTBuf, USBD_ENDP2_RXADDR, len);
                DevEP2_OUT_Deal( len );
                _SetEPRxStatus( EPindex, EP_RX_VALID );
			}
			if( wEPVal & EP_CTR_TX )
			{   
				/* ep2-IN */
                ClearEP_CTR_TX( EPindex );
            }
        }
        else if( EPindex == 0 )
        {          
			/* ep0 */
			if( wEPVal & EP_CTR_RX )
			{
				ClearEP_CTR_RX( ENDP0 );
                if( wEPVal & EP_SETUP )
                {    
					/* ep0-setup */
                    pBuf.b = PMAAddr + (uint8_t *)( _GetEPRxAddr( ENDP0 ) * 2 );
                    pSetupReqPakUSBD->bRequestType = *( pBuf.b + 0 );
                    pSetupReqPakUSBD->bRequest = *( pBuf.b + 1 );
                    pSetupReqPakUSBD->wValue = *( pBuf.w + 2 );
                    pSetupReqPakUSBD->wIndex = *( pBuf.w + 4 );
                    pSetupReqPakUSBD->wLength = *( pBuf.w + 6 );
                    SetupReqLen = pSetupReqPakUSBD->wLength;
                    SetupReqCode = pSetupReqPakUSBD->bRequest;
                    len=0;
                    if( ( pSetupReqPakUSBD->bRequestType & USB_REQ_TYP_MASK ) != USB_REQ_TYP_STANDARD )
                    {
                        /* ,, */
                        if( pSetupReqPakUSBD->bRequestType & 0x40 )             /*  */
                        {
                            switch( SetupReqCode )
                            {
                                case DEF_VEN_GET_VER:
                                    /* ȡоƬ汾 */
                                    buf[ 0 ] = 0x30;
                                    buf[ 1 ] = 0x00;                            /* (λ76ΪĴλ4Ϊ1/0ģʽ/ͨģʽλ3Ϊ1/0ȫMODEM/֣λ1λ0Ϊװ汾,λ0Ϊ1ʾQ,ΪL) */
                                   // buf[ 2 ] = 0x11;                            /* 5λΪоƬͺ(CH344̶Ϊ2),3λΪװ(000:L,001:Q) */
                                    pDescr = buf;
                                    len = 2;
                                    break;

                                case DEF_VEN_I2C_CMD_X:
                                    /* I2Cӿڵ,ִ */
                                    break;

                                case DEF_VEN_DELAY_MS:
                                    /* Ϊλʱָʱ */
                                    break;

                                case 0X95:
                                    /* Ĵ */
                                    buf[ 0 ] = 0x00;
                                    buf[ 1 ] = 0x00;                            /* (λ76ΪĴλ4Ϊ1/0ģʽ/ͨģʽλ3Ϊ1/0ȫMODEM/֣λ1λ0Ϊװ汾,λ0Ϊ1ʾQ,ΪL) */
                                   // buf[ 2 ] = 0x11;                            /* 5λΪоƬͺ(CH344̶Ϊ2),3λΪװ(000:L,001:Q) */
                                    pDescr = buf;
                                    len = 2;
                                    break;

                                case 0X9A:
                                    /* дĴ */
                                    break;

                                case DEF_VEN_UART0_INIT:
                                    /* ʼ0 */

                                case DEF_VEN_UART1_INIT:
                                    /* ʼ1 */

                                case DEF_VEN_UART2_INIT:
                                    /* ʼ2 */

                                case DEF_VEN_UART3_INIT:
                                    /* ʼ3 */
                                    break;

                                case DEF_VEN_UART0_M_OUT:
                                    /* MODEM0ź(BIT7-4: TXD/RTS/DTR/OUT) */
                                    break;

                                case DEF_VEN_GPIO_RD:
                                    /* GPIOȡ(GPIO11GPIO0ķ򼰵ƽ״̬) */
                                    break;

                                case DEF_VEN_GPIO_EN_WR:
                                    /* GPIOʹܼ/ */
                                    break;

                                case DEF_VEN_GPIO_IO_WR:
                                    /* GPIOƽ */
                                    break;

                                case DEF_VEN_GPIO_EN_RD:
                                    /* GPIOʹܼ/״̬ȡ */
                                    break;

                                default:
                                    errflag = 0xFF;                             /* ʧ */
                                    break;
                            }
                        }
                        else if( pSetupReqPakUSBD->bRequestType & 0x20 )        /*  */
                        {
                            switch( SetupReqCode )
                            {
                                case DEF_GET_LINE_CODING:
                                    pDescr = (UINT8 *)&UartCfg[ 0 ];
                                    len = 7;
                                    break;

                                case DEF_SET_LINE_CODING:

                                    break;

                                case DEF_SET_LINE_CTLSTE:
                                    /* RTS */
                                    break;

                                case DEF_SEND_BREAK:
                                    break;

                                default:
                                    errflag = 0xff;
                                    break;
                            }
                        }
                        if( errflag != 0xFF )
                        {
                            if( SetupReqLen > len )
                            {
                                SetupReqLen = len;
                            }
                            len = ( SetupReqLen >= DevEP0SIZE ) ? DevEP0SIZE : SetupReqLen;
                            UserToUSBDBufferCopy( pDescr, USBD_ENDP0_TXADDR, len );
                            pDescr += len;
                        }

                    }
                    else
                    {
                        switch( pSetupReqPakUSBD->bRequest )
                        {
                            case USB_GET_DESCRIPTOR:
                            {
                                switch( ( ( pSetupReqPakUSBD->wValue ) >> 8 ) )
                                {
                                    case USB_DESCR_TYP_DEVICE:
                                        pDescr = (uint8_t *)USBD_DeviceDescriptor;
                                        len = USBD_DeviceDescriptor[ 0 ];
                                        break;
                                        
                                    case USB_DESCR_TYP_CONFIG:
                                        pDescr = (uint8_t *)USBD_ConfigDescriptor;
                                        len = USBD_ConfigDescriptor[ 2 ];
                                        break;
                                        
                                    case USB_DESCR_TYP_STRING:
                                         /* ȡַ */
                                         switch( ( pSetupReqPakUSBD->wValue ) & 0xff )
                                         {
                                             case 0:
                                                 /* ַ */
                                                 pDescr = (uint8_t *)USBD_StringLangID;
                                                 len = USBD_StringLangID[ 0 ];
                                                 break;

                                             case 1:
                                                 /* USBַ */
                                                 pDescr = (uint8_t *)USBD_StringVendor;
                                                 len = USBD_StringVendor[ 0 ];
                                                 break;

                                             case 2:
                                                 /* USBƷַ */
                                                 pDescr = (uint8_t *)USBD_StringProduct;
                                                 len = USBD_StringProduct[ 0 ];
                                                 break;


                                             default:
                                                 errflag = 0xFF;
                                                 break;
                                         }
                                         break;



                                    default:
                                        errflag = 0xff;
                                        break;
								}
                                if( errflag != 0xFF )
                                {
                                    if( SetupReqLen > len ) 
                                    {
                                    	SetupReqLen = len;
                                    }
                                    len = ( SetupReqLen >= DevEP0SIZE ) ? DevEP0SIZE : SetupReqLen;
                                    UserToUSBDBufferCopy( pDescr, USBD_ENDP0_TXADDR, len );
                                    pDescr += len;
                                }
                            }
                            break;

                            case USB_SET_ADDRESS:
                                DevAddress = ( pSetupReqPakUSBD->wValue ) & 0xff;
                                break;

                            case USB_SET_CONFIGURATION:
                                DevConfig = ( pSetupReqPakUSBD->wValue ) & 0xff;
                                break;

                            default:
                                errflag = 0xFF;
                                break;
                        }
                    }
                    if( errflag == 0xff )
                    {
                        _SetEPRxTxStatus( ENDP0, EP_RX_STALL, EP_TX_STALL );     // data1+stall
                    }
                    else
                    {
                        if( pSetupReqPakUSBD->bRequestType & 0x80 )
                        {
                            len = (SetupReqLen > DevEP0SIZE) ? DevEP0SIZE : SetupReqLen;
                            SetupReqLen -= len;
                        }
                        else  
                        {
                        	len = 0;
                        }
                        SetEPTxCount( ENDP0, len );
                        _SetEPRxTxStatus( ENDP0, EP_RX_VALID, EP_TX_VALID );     // data1+ACK
                    }
                }
                else
                {                       
                	// ep0-OUT */
                    if( SetupReqCode == DEF_SET_LINE_CODING )
                    {
                        /* 洮ڲʵز */
                        ClearEP_CTR_RX( ENDP0 );
                        len = GetEPRxCount( ENDP0 );
                        USBDToUserBufferCopy( UartCfg, USBD_ENDP2_RXADDR, len );
                    }

                    SetEPRxStatus( ENDP0 , EP_RX_VALID );
                }
                return;
            }
            if( wEPVal & EP_CTR_TX )
            {       
            	// ep0-IN */
                ClearEP_CTR_TX( ENDP0 );
                switch( pSetupReqPakUSBD->bRequest )
                {
	                case USB_GET_DESCRIPTOR:
	                    len = SetupReqLen >= DevEP0SIZE ? DevEP0SIZE : SetupReqLen;
	                    UserToUSBDBufferCopy( pDescr, USBD_ENDP0_TXADDR, len );
	                    SetupReqLen -= len;
	                    pDescr += len;
	                    SetEPTxCount( ENDP0, len );     // AUTO turn tog */
	                    SetEPTxStatus( ENDP0 , EP_TX_VALID );
	                    break;
	                    
	                case USB_SET_ADDRESS:
	                    SetDADDR( DevAddress | DADDR_EF );
	                    SetEPTxStatus( ENDP0 , EP_TX_NAK );
	                    break;
	                    
	                default:
	                    SetEPTxStatus( ENDP0 , EP_TX_NAK );
	                    break;
                }
                return;
            }
        }
    }
	if( intflag & ISTR_RESET )
	{ 
		/* RESET (clear-only bit) */
        SetISTR( (uint16_t)CLR_RESET );
        SetBTABLE( USBD_BTABLE_ADDRESS );
        USBD_Endp_Init( );
        _SetDADDR( 0 | DADDR_EF ); 
	}
	if( intflag & ISTR_SUSP )
	{ 
		/* SUSPend (clear-only bit) */
        SetISTR( (uint16_t)CLR_SUSP );
        wCNTR = GetCNTR( );
        wCNTR |= CNTR_FSUSP;
        SetCNTR( wCNTR );
        wCNTR = GetCNTR( );
        wCNTR |= CNTR_LPMODE;
        SetCNTR( wCNTR );
    }
	if( intflag & ISTR_WKUP )
	{ 
		/* WaKe UP (clear-only bit) */
        SetISTR( (uint16_t)CLR_WKUP );
        wCNTR = GetCNTR( );
        wCNTR &= ~( CNTR_LPMODE | CNTR_FSUSP );
        SetCNTR( wCNTR );
    }
	if( intflag & ISTR_ERR )
	{ 
		/* ERRor (clear-only bit) */
        SetISTR( (uint16_t)CLR_ERR );
    }
}

/*******************************************************************************
* Function Name  : DevEP2_OUT_Deal
* Description    : Deal device Endpoint 2 OUT.
* Input          : l: Data length.
* Return         : None
*******************************************************************************/
void DevEP2_OUT_Deal( UINT8 l )
{
    UINT8 i,j;
    UINT16 AddSum;
    for(i=0; i<l; i++)
    {
        if(sendUsbData.recDataLen == 0)
        {
            if(USBD_pEP2_OUTBuf[i] == 0xAA)
            {
                sendUsbData.recDataBuf[0] = USBD_pEP2_OUTBuf[i];
                sendUsbData.recDataLen = 1;
            }
        }
        else if(sendUsbData.recDataLen == 1)
        {
            if(USBD_pEP2_OUTBuf[i] == 0x55)
            {
                sendUsbData.recDataBuf[1] = USBD_pEP2_OUTBuf[i];
                sendUsbData.recDataLen = 2;
            }
            else
            {
                sendUsbData.recDataLen = 0;
            }
        }
        else if(sendUsbData.recDataLen == 2)
        {
            if(USBD_pEP2_OUTBuf[i] <= 16)     //---֡---
            {
                sendUsbData.recDataBuf[2] = USBD_pEP2_OUTBuf[i];
                sendUsbData.recDataLen = 3;
            }
            else
            {
                sendUsbData.recDataLen = 0;
            }
        }
        else
        {
            sendUsbData.recDataBuf[sendUsbData.recDataLen] = USBD_pEP2_OUTBuf[i];
            sendUsbData.recDataLen++;
            if(sendUsbData.recDataLen >= sendUsbData.recDataBuf[2])
            {
                AddSum = 0;
                for(j = 0; j < sendUsbData.recDataBuf[2]-1; j++)
                {
                    AddSum += sendUsbData.recDataBuf[j];
                }
                sendUsbData.recDataLen = 0;
                if(sendUsbData.recDataBuf[sendUsbData.recDataBuf[2]-1] == (AddSum&0x00FF))   //---Уȷʶʲô---
                {
                    switch(sendUsbData.recDataBuf[3])
                    {
                        case 0x01:      //---ȡͨϴ--- sendDataP---
                            AddSum = 0;
                            sendUsbData.recDataBuf[3] = 0x81;
                            for(j = 0; j < sendUsbData.recDataBuf[2]-1; j++)
                            {
                                USBD_pEP2_INBuf[j] = sendUsbData.recDataBuf[j];
                                AddSum += sendUsbData.recDataBuf[j];
                            }
                            USBD_pEP2_INBuf[sendUsbData.recDataBuf[2]-1] = (AddSum&0x00FF);
                            //--------
//                            R8_UEP2_T_LEN = sendUsbData.recDataBuf[2];
//                            R8_UEP2_CTRL = (R8_UEP2_CTRL & ~MASK_UEP_T_RES)| UEP_T_RES_ACK;

                            UserToUSBDBufferCopy(USBD_pEP2_INBuf,USBD_ENDP2_TXADDR,sendUsbData.recDataBuf[2]);
                            SetEPTxCount(ENDP2, sendUsbData.recDataBuf[2]);
                            SetEPTxValid(ENDP2);

                            //------ҪʱԱ֤ݴͷ0x81ָͻ------
                            sendUsbData.delayNum = 250;
                            sendUsbData.sendDataLen = sendUsbData.recDataBuf[2] - 5;
                            for(j = 0; j < MaxUsbDataLen; j++)
                            {
                                sendUsbData.sendDataType[j] = 0;
                            }
                            for(j = 0; j < sendUsbData.sendDataLen; j++)
                            {
                                sendUsbData.sendDataType[j] = sendUsbData.recDataBuf[j+4]-1;
                            }
                            //sendUsbData.finishFlag = 1;
                            break;
                        case 0x02:      //---رϴ---
                            sendUsbData.sendDataLen = 0;
                            break;
                    }
                    break;     //---ǰiѭ----
                }
            }
        }
    }
}



UINT16 getUsbBufLen(void)
{
    UINT8 reDataLen;
    if(sendUsbData.head >= sendUsbData.end)
    {
        reDataLen = sendUsbData.head - sendUsbData.end;
    }
    else
    {
        reDataLen = (UINT16)USB_SENDBUF_LEN - sendUsbData.end + sendUsbData.head;
    }
    return reDataLen;
}
void deal_sendMonitorData(void)
{
    UINT8 i, frameByteNum;
        UINT16 data;
        uint32_t *pdwVal;
        pdwVal = (uint32_t *)(USBD_ENDP2_TXADDR * 2 + PMAAddr);

        if(sendUsbData.delayNum == 0)
        {
            if(sendUsbData.sendDataLen)
            {
                if(getUsbBufLen() >= needSendLen[sendUsbData.sendDataLen])
                {
                    if( _GetEPTxStatus(ENDP2) == EP_TX_NAK )
                    {
                        frameByteNum = 4;
                        while(frameByteNum < usbFrameByteNum[sendUsbData.sendDataLen])
                        {
                            for(i = 0; i < sendUsbData.sendDataLen; i++)
                            {
                                *(pdwVal+(frameByteNum+1)/2)=sendUsbData.sendDataBuf[sendUsbData.end+i];
                                frameByteNum+=2;
                            }
                            sendUsbData.end += MaxUsbDataLen;
                        }
                        sendUsbData.heartBeat++;
                        *pdwVal=(uint16_t)0x55AA;
                        data=sendUsbData.heartBeat<<8;
                        data|=usbFrameByteNum[sendUsbData.sendDataLen];
                        *(pdwVal+1)= data;

                        SetEPTxCount(ENDP2, usbFrameByteNum[sendUsbData.sendDataLen]);
                        SetEPTxValid(ENDP2);
                    }
                }
            }
        }
    else
    {
        sendUsbData.delayNum--;
    }
}
